    @echo off
    setlocal enabledelayedexpansion
    set IBM_SP_SCRIPT_VERSION=2.3
    REM **************************************************************************
    REM * build_recovery_image.cmd
    REM *
    REM * (c) Copyright by IBM Corporation and other(s) 2018, 2021. All Rights
    REM * Reserved.
    REM *
    REM * This script builds a WinPE image with an embedded IBM Spectrum Protect
    REM * Backup-Archive Client. The user must download a particular client image
    REM * that will be embedded, and then pass the name of the image to the
    REM * script.
    REM *
    REM * This script is provided "as is", without warranty of any kind.
    REM *
    REM * Author: Andrew M. Raibeck
    REM *         IBM Spectrum Protect Development
    REM *         IBM Systems, Storage
    REM *         storman@us.ibm.com
    REM *
    REM * Version 2.3 - 2022-01-04 - Resequence the version history in most-to-
    REM *                            least recent order.
    REM *                          - Reposition the IBM_SCRIPT_VERSION environment
    REM *                            variable setting to the top of the script (to
    REM *                            make it easier to remember to update it!).
    REM *
    REM * Version 2.2 - 2021-08-30 - Change version number to 2.2 to align with
    REM *                            updated restore_system.cmd script. Fixed
    REM *                            copyright date in the version display info.
    REM *
    REM * Version 2.1 - 2021-02-16 - Minor (nit) bug fix, up the version number
    REM *                            to 2.1.
    REM *
    REM * Version 1.6 - 2021-01-14 - If dsmcert.kdb and dsmcert.sth are present,
    REM *                            add them to the baclient directory. Modify
    REM *                            startnet.cmd to ask the user whether to start
    REM *                            the restore script after WinPE boots.
    REM *
    REM *
    REM * Version 1.5 - 2020-12-02 - Added unique dummy file to root of the media
    REM *                            directory. Added restore_system.cmd script
    REM *                            to the media dirctory. Added WSH and
    REM *                            PowerShell scripting capability to the WinPE
    REM *                            image. Script will copy itself to the
    REM *                            WinPE system drive, then re-launch it from
    REM *                            that drive. Show instructions for writing
    REM *                            the WinPE image to a USB thumb drive.
    REM * Version 1.4 - 2019-11-07 - Fix the deletion of the common and common64
    REM *                            directories so as not to delete the gsk8
    REM *                            subdirectory under common and common64.
    REM *
    REM * Version 1.3 - 2019-07-23 - Bug fixes and other improvements:
    REM *                               - Call to the DISM exit script fails when
    REM *                                 the script is not on the same drive as
    REM *                                 the ADK.
    REM *                               - The -debug option did not work.
    REM *                               - Add DISM exit script name to logging.
    REM *                               - Delete the WinRoot directory introduced
    REM *                                 in Version 8.1.7. This directory has the
    REM *                                 web server used by the web client, but
    REM *                                 is not needed in WinPE mode.
    REM *
    REM * Version 1.2 - 2018-11-13 - Add a user exit that provides for injection
    REM *                            of custom features and drivers.
    REM *
    REM * Version 1.1 - 2018-06-29 - Initial version
    REM **************************************************************************

    REM **************************************************************************
    REM * Beginning of user-customizable settings
    REM **************************************************************************
    REM This can be configured by the user. This is the base directory for
    REM creating the WinPE image.
    set IBM_SP_RECOVERY_DIR=C:\ibm_sp_recovery
    REM **************************************************************************
    REM * End of user-customizable settings
    REM **************************************************************************

    echo.
    echo -------------------------------------------------------------------------------
    echo IBM Spectrum Protect
    echo build_recovery_image.cmd version %IBM_SP_SCRIPT_VERSION%
    echo (c) Copyright by IBM Corporation and other(s) 2018, 2021. All Rights Reserved.
    echo -------------------------------------------------------------------------------
    echo.

    REM %0 is the full path used to invoke the script.
    REM %~nx0 gets just the file name and extension of the script.
    REM %~p0 gets the directory that contains the script file.
    set IBM_SP_SCRIPT_NAME=%~nx0
    set IBM_SP_SCRIPT_DIR=%~p0
    set IBM_SP_SCRIPT_DRIVE=%~d0

    REM Set up dsmcert file variables.
    set IBM_SP_DSMCERT_KDB=%IBM_SP_SCRIPT_DRIVE%%IBM_SP_SCRIPT_DIR%dsmcert.kdb
    set IBM_SP_DSMCERT_STH=%IBM_SP_SCRIPT_DRIVE%%IBM_SP_SCRIPT_DIR%dsmcert.sth

    REM An exit script where users can add their own custom packages or drivers.
    set IBM_SP_DISMEXIT_SCRIPT_NAME=build_recovery_image_dismexit.cmd

    REM Version 1.3: Build full path to the DISM exit script
    set IBM_SP_DISMEXIT_SCRIPT_FULL_PATH=%IBM_SP_SCRIPT_DRIVE%%IBM_SP_SCRIPT_DIR%%IBM_SP_DISMEXIT_SCRIPT_NAME%

    REM Version 1.6: Set up dsmcert file variables.
    set IBM_SP_DSMCERT_KDB=%IBM_SP_SCRIPT_DRIVE%%IBM_SP_SCRIPT_DIR%dsmcert.kdb
    set IBM_SP_DSMCERT_STH=%IBM_SP_SCRIPT_DRIVE%%IBM_SP_SCRIPT_DIR%dsmcert.sth

    REM Version 1.5: Include the restore script
    set IBM_SP_RESTORE_SYSTEM_SCRIPT=restore_system.cmd
    set IBM_SP_RESTORE_SYSTEM_SCRIPT_FULL_PATH=%IBM_SP_SCRIPT_DRIVE%%IBM_SP_SCRIPT_DIR%%IBM_SP_RESTORE_SYSTEM_SCRIPT%

    REM Version 1.5: Dummy file to help uniquely identify the WinPE media drive
    set IBM_SP_DUMMY_ID_FILE=_____I_AM_WINPE_BOOT_MEDIA_WITH_IBM_SPECTRUM_PROTECT_CLIENT______

    REM How many ways can a user ask for help?
    REM A question mark (?)
    if "%1" == "?" goto :USAGE
    if "%1" == "-?" goto :USAGE
    if "%1" == "--?" goto :USAGE
    if "%1" == "/?" goto :USAGE
    REM The letter 'h'
    if /i "%1" == "h" goto :USAGE
    if /i "%1" == "-h" goto :USAGE
    if /i "%1" == "--h" goto :USAGE
    if /i "%1" == "/h" goto :USAGE
    REM The word 'help'
    if /i "%1" == "help" goto :USAGE
    if /i "%1" == "-help" goto :USAGE
    if /i "%1" == "--help" goto :USAGE
    if /i "%1" == "/help" goto :USAGE

    REM The word 'usage'
    if /i "%1" == "usage" goto :USAGE
    if /i "%1" == "-usage" goto :USAGE
    if /i "%1" == "--usage" goto :USAGE
    if /i "%1" == "/usage" goto :USAGE

    REM Version 1.6: Scan the input arguments for "-debug" or "-addcert"
    set IBM_SP_DEBUG=false
    set IBM_SP_ADDCERT=false

:SCAN_OPTIONS
    set IBM_SP_CONTINUE_SCAN=false

    if /i "%1" equ "-debug" (
        set IBM_SP_DEBUG=true
        set IBM_SP_CONTINUE_SCAN=true
        shift
    ) else (
        if /i "%1" equ "-addcert" (
            set IBM_SP_ADDCERT=true
            set IBM_SP_CONTINUE_SCAN=true
            shift
        )
    )

    if "%IBM_SP_CONTINUE_SCAN%" equ "true" goto :SCAN_OPTIONS

    if "%IBM_SP_DEBUG%" equ "true" (@echo on)

    REM Expected input is a downloaded backup-archive client install package name.
    REM Make sure there are no extra arguments.
    REM Make sure the client package exists.
    if "%1" equ "" goto :PACKAGE_NOT_SPECIFIED
    if "%2" neq "" goto :TOO_MANY_ARGS

    REM Make sure the specified package exists.
    if not exist "%1" goto :PACKAGE_NOT_FOUND

    REM Get the path of the package name. This is necessary because later, when
    REM the ADK command shell initialization script DandISEtEnv.bat is called,
    REM the current directory will be changed; thus if the input package name is
    REM a relative path name, this script will not be able to find it after the
    REM current directory is changed.
    set IBM_SP_FULL_PKG_NAME=%~f1

    REM Verify that the DISM user exit can be found. This file must exist, even
    REM if the user does not modify it.
    if not exist "%IBM_SP_DISMEXIT_SCRIPT_FULL_PATH%" goto :DISMEXIT_NOT_FOUND

    REM Version 1.6: Verify that the restore script can be found. This file must exist.
    if not exist "%IBM_SP_RESTORE_SYSTEM_SCRIPT_FULL_PATH%" goto :RESTORE_SCRIPT_NOT_FOUND

    REM If -addcert was specified, make sure the certification files are in the
    REM current directory.
    set IBM_SP_CERTFILES_FOUND=true

    if "%IBM_SP_ADDCERT%" equ "true" (
        if not exist "%IBM_SP_DSMCERT_KDB%" (
            set IBM_SP_CERTFILES_FOUND=false
            echo ERROR: -addcert was specified, but "%IBM_SP_DSMCERT_KDB%" is not found.
        )

        if not exist "%IBM_SP_DSMCERT_STH%" (
            set IBM_SP_CERTFILES_FOUND=false
            echo ERROR: -addcert was specified, but "%IBM_SP_DSMCERT_STH%" is not found.
        )

        if "!IBM_SP_CERTFILES_FOUND!" equ "false" goto :CERT_FILES_NOT_FOUND
    )

    REM Verify that we are running as an administrator with elevated privileges.
    whoami /groups | find "S-1-16-12288" > nul 2>&1
    if %errorlevel% neq 0 goto :NOT_ELEVATED_ADMIN

    REM See if the ADK 10 is installed. Working assumption is that the kit is
    REM installed if all of the following are true:
    REM
    REM   - The KitsRoot10 registry value, which is the root directory for the
    REM     ADK, can be found.
    REM
    REM   - The ADK deployment tools directory can be found.
    REM
    REM   - The DandISetEnv.bat file, used to initialize the ADK command shell,
    REM     can be found.
    REM
    REM Note: The general logic for locating the above information is based on
    REM the first parts of the DandISetEnv.bat file. This information is subject
    REM to change at the discretion of Microsoft, and can change from one ADK
    REM version to the next.
    set IBM_SP_REG_KIT_KEY=HKLM\Software\Wow6432Node\Microsoft\Windows Kits\Installed Roots
    set IBM_SP_REG_KIT_VALUE=KitsRoot10
    set IBM_SP_REG_KIT_KEY_FOUND=false
    set /a IBM_SP_COUNT=0

    echo.
    echo -------------------------------------------------------------------------------
    echo - Looking for the Windows 10 ADK Deployment Tools directory
    echo -------------------------------------------------------------------------------

:FIND_DEPLOYMENT_TOOLS_DIR
    set /a IBM_SP_COUNT+=1
    if %IBM_SP_COUNT% gtr 2 goto :DEPLOY_TOOLS_NOT_INSTALLED

    reg query "%IBM_SP_REG_KIT_KEY%" /v %IBM_SP_REG_KIT_VALUE% >nul 2>&1 && set IBM_SP_REG_KIT_KEY_FOUND=true

    if "%IBM_SP_REG_KIT_KEY_FOUND%" equ "false" (
        set IBM_SP_REG_KIT_KEY=HKLM\Software\Microsoft\Windows Kits\Installed Roots
        goto :FIND_DEPLOYMENT_TOOLS_DIR
    )

    for /f "skip=2 tokens=2*" %%a in ('reg query "%IBM_SP_REG_KIT_KEY%" /v %IBM_SP_REG_KIT_VALUE%') do (
        set IBM_SP_DEPLOYMENT_TOOLS_INIT_SCRIPT=%%bAssessment and Deployment Kit\Deployment Tools\DandISetEnv.bat
    )

    if not exist "%IBM_SP_DEPLOYMENT_TOOLS_INIT_SCRIPT%" goto :DEPLOY_TOOLS_INIT_NOT_FOUND

    echo.
    echo -------------------------------------------------------------------------------
    echo - The Windows 10 ADK Deployment Tools directory was found
    echo -------------------------------------------------------------------------------

    REM Now try to determine if we are already running in the Deployment Tools
    REM command shell. If not, run the Deployment tools command shell
    REM initialization script.
    set /a IBM_SP_COUNT=0
    set /a IBM_SP_NUM_TESTS=6

:CHECK_IF_DEPLOY_TOOLS_SHELL_IS_RUNNING
    set /a IBM_SP_COUNT+=1
    set /a IBM_SP_NUM_TESTS_PASSED=0
    if %IBM_SP_COUNT% gtr 2 goto :CANNOT_INITIALIZE_DEPLOYMENT_TOOLS

    echo.
    echo -------------------------------------------------------------------------------
    echo - Checking to see if the Deployment Tools environment is initialized
    echo -------------------------------------------------------------------------------

    REM %DandIRoot% is set by the Deployment Tools command shell script, and
    REM matches the current directory immediately after launch.
    if /i "%cd%" == "%DandIRoot%" set /a IBM_SP_NUM_TESTS_PASSED+=1

    REM Check if the ADK version is for Windows 10.
    if /i "%KitsRootRegValueName%" == "KitsRoot10" set /a IBM_SP_NUM_TESTS_PASSED+=1

    if exist "..\Windows Preinstallation Environment" set /a IBM_SP_NUM_TESTS_PASSED+=1

    REM   dism returns 0 when no arguments are passed.
    dism >nul 2>&1 && set /a IBM_SP_NUM_TESTS_PASSED+=1

    REM   copype returns 1 when no arguments are passed.
    call copype >nul 2>&1
    if %ERRORLEVEL% equ 1 set /a IBM_SP_NUM_TESTS_PASSED+=1

    REM   makewinpemedia returns 1 when no arguments are passed.
    call makewinpemedia >nul 2>&1
    if %ERRORLEVEL% equ 1 set /a IBM_SP_NUM_TESTS_PASSED+=1

    if %IBM_SP_NUM_TESTS_PASSED% equ %IBM_SP_NUM_TESTS% (
        REM If the number of tests passed matches the total number of tests,
        REM then we'll assume the ADK command shell is initialized.
        echo.
        echo -------------------------------------------------------------------------------
        echo - The Deployment Tools appears to be initialized
        echo -------------------------------------------------------------------------------
        goto :UNPACK_CLIENT_INSTALL_IMAGE

    ) else (
        REM The ADK does not seem to be initialized, at least not fully. Call
        REM the script initialization, then re-run the pre-flight check.
        echo.
        echo -------------------------------------------------------------------------------
        echo - The Deployment Tools does not seem to be initialized; initializing now
        echo -------------------------------------------------------------------------------
        call "%IBM_SP_DEPLOYMENT_TOOLS_INIT_SCRIPT%"
        REM The Deployment Tools initialization script might turn off echo, so turn it back
        REM on again when debug is enabled.
        if "%IBM_SP_DEBUG%" == "true" @echo on
        goto :CHECK_IF_DEPLOY_TOOLS_SHELL_IS_RUNNING
    )

:UNPACK_CLIENT_INSTALL_IMAGE
    REM Identify the version number, then create a temporary directory for the
    REM extracted install image. The '~' strips off surrounding quotation marks.
    for %%f in ("%IBM_SP_FULL_PKG_NAME%") do (
        set IBM_SP_PKG_NAME=%%~nxf
    )

    REM Remove any quotation marks from the name, if it is quoted. I could have
    REM used the %~1 form earlier, but that only works if there are quotes on
    REM both sides of the file name. However, %~1 does not work if the quotes
    REM are on the left side only, which is unbalanced but accepted by Windows.
    set IBM_SP_PKG_NAME=%IBM_SP_PKG_NAME:"=%

    for /f "tokens=1-8 delims=.-" %%a in ("%IBM_SP_PKG_NAME%") do (
        set IBM_SP_VER=%%a
        set IBM_SP_REL=%%b
        set IBM_SP_MAINT=%%c
        set IBM_SP_IF=%%d
        set IBM_SP_TIV=%%e
        set IBM_SP_TSMBAC=%%f
        set IBM_SP_ARCH=%%g
        set IBM_SP_EXT=%%h
    )

    REM Check that the package name is of the expected format, and that the
    REM package version is supported by this script. The expected format of the
    REM package name is "<v>.<r>.<m>.<f>-TIV-TSMBAC-WinX<nn>.exe" where '<v>' is
    REM the version, '<r>' is the release, '<m>' is the maintenance (fix pack)
    REM level, '<f>' is the interim fix level, '<nn>' is 32 or 64 (processor
    REM architecture), and the other parts are literal text. This script
    REM supports backup-archive client 7.1.6.0 and later interim fixes, fix
    REM packs, releases, and versions.
    if /i not "%IBM_SP_TIV%" == "TIV" goto :BAD_PACKAGE_NAME
    if /i not "%IBM_SP_TSMBAC%" == "TSMBAC" goto :BAD_PACKAGE_NAME
    if /i not "%IBM_SP_EXT%" == "EXE" goto :BAD_PACKAGE_NAME
    if /i "%IBM_SP_ARCH%" == "WinX32" (
        set IBM_SP_WINPE_ARCH=x86
    ) else if /i "%IBM_SP_ARCH%" == "WinX64" (
        set IBM_SP_WINPE_ARCH=amd64
    ) else goto :BAD_PACKAGE_NAME

    set IBM_SP_IS_SECVER=false
    if %IBM_SP_VER% equ 7 (
        if %IBM_SP_REL% neq 1 goto :BAD_VERSION
        if %IBM_SP_MAINT% lss 6 goto :BAD_VERSION
        if %IBM_SP_MAINT% geq 8 set IBM_SP_IS_SECVER=true
    ) else if %IBM_SP_VER% equ 8 (
        if %IBM_SP_REL% neq 1 goto :BAD_VERSION
        if %IBM_SP_MAINT% geq 2 set IBM_SP_IS_SECVER=true
    ) else goto :BAD_VERSION

    set IBM_SP_PKG_DIR=%IBM_SP_RECOVERY_DIR%\ibm_sp_images\ibm_sp_client_%IBM_SP_VER%%IBM_SP_REL%%IBM_SP_MAINT%%IBM_SP_IF%_%IBM_SP_WINPE_ARCH%
    if exist "%IBM_SP_PKG_DIR%" (
    echo.
    echo -------------------------------------------------------------------------------
    echo - Deleting directory "%IBM_SP_PKG_DIR%"
    echo -------------------------------------------------------------------------------
        rd /s /q "%IBM_SP_PKG_DIR%"
        if %ERRORLEVEL% neq 0 goto :COULD_NOT_CLEAN_IBMSPPKGDIR
        if exist "%IBM_SP_PKG_DIR%" goto :COULD_NOT_CLEAN_IBMSPPKGDIR
    )

    REM Extract the install image
    echo.
    echo -------------------------------------------------------------------------------
    echo - Extracting installation image to "%IBM_SP_PKG_DIR%" with command
    echo - "%IBM_SP_FULL_PKG_NAME%" -o"%IBM_SP_PKG_DIR%"
    echo -------------------------------------------------------------------------------
    "%IBM_SP_FULL_PKG_NAME%" -o"%IBM_SP_PKG_DIR%"
    set IBM_SP_LAST_RC=%ERRORLEVEL%
    echo return code from extract is %IBM_SP_LAST_RC%
    if %IBM_SP_LAST_RC% neq 0 goto :EXTRACT_FAILED

    set IBM_SP_WINPE_DIR=%IBM_SP_RECOVERY_DIR%\WinPE_%IBM_SP_VER%%IBM_SP_REL%%IBM_SP_MAINT%%IBM_SP_IF%_%IBM_SP_WINPE_ARCH%
    set IBM_SP_WINPE_IMAGE=%IBM_SP_RECOVERY_DIR%\WinPE_%IBM_SP_VER%%IBM_SP_REL%%IBM_SP_MAINT%%IBM_SP_IF%_%IBM_SP_WINPE_ARCH%.iso

    REM Initialize the WinPE image.
    echo.
    echo -------------------------------------------------------------------------------
    echo - Initializing the WinPE image directory "%IBM_SP_WINPE_DIR%"
    echo -------------------------------------------------------------------------------
    if exist "%IBM_SP_WINPE_DIR%" rd /s /q "%IBM_SP_WINPE_DIR%"
    if exist "%IBM_SP_WINPE_IMAGE%" del "%IBM_SP_WINPE_IMAGE%"

    call copype %IBM_SP_WINPE_ARCH% "%IBM_SP_WINPE_DIR%"
    set IBM_SP_LAST_RC=%ERRORLEVEL%
    if %IBM_SP_LAST_RC% neq 0 goto :COPY_PE_FAILED

    REM Make sure the WinPE image dir was created (the above rc check should
    REM suffice, but better to do the sanity check).
    if not exist "%IBM_SP_WINPE_DIR%\media" goto :WINPE_DIR_NOT_CREATED

    REM run dsmc-pe tool to set up the client for running in a WinPE setting.
    REM Temporarily change to the client directory inside the unpacked install image.
    echo.
    echo -------------------------------------------------------------------------------
    echo - Set up baclient directory for running in WinPE mode
    echo -------------------------------------------------------------------------------
    pushd "%IBM_SP_PKG_DIR%\TSMClient"
    if %ERRORLEVEL% neq 0 goto :COULD_NOT_CHANGE_TO_TSMCLIENT_DIR

    if "%IBM_SP_WINPE_ARCH%" == "amd64" (
        call dsmc-pe-x64 set
    ) else (
        REM Work around the bug in dsmc-pe.cmd that refers to gsk8\lib64 instead of
        REM gsk8\lib.
        findstr /i /c:"SET GSKRUNTIME=.\Program Files\ibm\gsk8\lib64" dsmc-pe.cmd
        if %ERRORLEVEL% equ 0 (
            rename "program files\ibm\gsk8\lib" lib64
            call dsmc-pe set
            rename "program files\ibm\gsk8\lib64" lib
        ) else (
            call dsmc-pe set
        )
    )

    REM Clean up unnecessary files in order to optimize the size of the recovery
    REM image. Beginning with the 8.1 client, IBM Java is included in the
    REM ibmspjvm directory. Before 7.1, IBM Java was installed in a directory
    REM named jvmnnnnn (nnnnn varies) in the baclient directory.
    echo.
    echo -------------------------------------------------------------------------------
    echo - Clean up unnecessary files in the client install image
    echo -------------------------------------------------------------------------------
    del *.ini *.mst *.msi *.htm *.exe *.spd *.cmd
    rd /s /q commonappdata issetupprerequisites win windows
    if exist ibmspjvm rd /s /q ibmspjvm

    REM Version 1.3: Get rid of the web server introduced in client version 8.1.7
    if exist winroot rd /s /q winroot

    if "%IBM_SP_WINPE_ARCH%" == "amd64" (
        if exist system64 rd /s /q system64
        if exist "Program Files 64\Tivoli\TSM\baclient\cit" rd /s /q "Program Files 64\Tivoli\TSM\baclient\cit"
        for /d %%a in ("program files 64\tivoli\tsm\baclient\jvm*") do rd /s /q "%%a"
    ) else (
        if exist system32 rd /s /q system32
        if exist "Program Files\Tivoli\TSM\baclient\cit" rd /s /q "Program Files\Tivoli\TSM\baclient\cit"
        for /d %%a in ("program files\tivoli\tsm\baclient\jvm*") do rd /s /q "%%a"
    )

    if "%IBM_SP_IS_SECVER%" == "false" (
        REM If this is not the secure client code introduced in versions 7.1.8
        REM and 8.1.2, then the entire common[64] directory can be deleted.
        if "%IBM_SP_WINPE_ARCH%" == "amd64" (
            if exist common64 rd /s /q common64
        ) else (
            if exist common rd /s /q common
        )
    ) else (
        REM For the secure client versions, delete only selected directories.
        if "%IBM_SP_WINPE_ARCH%" == "amd64" (
            if exist common rd /s /q common
            cd common64\tivoli\tsm\api64
        ) else (
            cd common\tivoli\tsm\api
        )

        del /q *

        if exist ..\doc rd /s /q ..\doc
        if exist ..\license rd /s /q ..\license
        if exist cit rd /s /q cit
        if exist swidtag rd /s /q swidtag
        if exist gsk8\catalogs rd /s /q gsk8\catalogs
        if exist gsk8\gskssl64 rd /s /q gsk8\gskssl64
    )

    echo.
    echo -------------------------------------------------------------------------------
    echo - Copying install image from "%IBM_SP_PKG_DIR%" to "%IBM_SP_WINPE_DIR%\media"
    echo -------------------------------------------------------------------------------
    robocopy "%IBM_SP_PKG_DIR%\TSMClient" "%IBM_SP_WINPE_DIR%\media" /e
    set IBM_SP_LAST_RC=%ERRORLEVEL%

    REM RC 1 means that all files were copied as expected. However because
    REM the target directory (..\media) already exists, the return code will
    REM probably be 3, which means that some files were copied but additional
    REM files were present; no copy failure was encountered.
    REM Other return code information can be found here:
    REM https://support.microsoft.com/en-us/help/954404/return-codes-that-are-used-by-the-robocopy-utility-in-windows-server-2
    if %IBM_SP_LAST_RC% neq 1 (
        if %IBM_SP_LAST_RC% neq 3 goto :ROBOCOPY_FAILED
    )

    REM Version 1.5: Put a "dummy" file in the media directory to help identify
    REM the WinPE media drive when booted into WinPE. Unfortunately WinPE
    REM provides no mechanism to readily identify the drive used to boot WinPE.
    REM This file should have a sufficiently unique name.
    echo This is the WinPE boot drive with the IBM Spectrum Protect client>"%IBM_SP_WINPE_DIR%\media\%IBM_SP_DUMMY_ID_FILE%"

    REM Version 1.5: Copy the restore script to the media directory
    if exist "%IBM_SP_SCRIPT_DRIVE%\%IBM_SP_SCRIPT_DIR%\%IBM_SP_RESTORE_SYSTEM_SCRIPT%" copy "%IBM_SP_SCRIPT_DRIVE%\%IBM_SP_SCRIPT_DIR%\%IBM_SP_RESTORE_SYSTEM_SCRIPT%" "%IBM_SP_WINPE_DIR%\media\%IBM_SP_RESTORE_SYSTEM_SCRIPT%"

    REM Version 1.6: Add dsmcert.kdb and dsmcert.sth files if -addcert is specified.
    if "%IBM_SP_ADDCERT%" equ "true" (
        if "%IBM_SP_WINPE_ARCH%" == "amd64" (
            copy "%IBM_SP_DSMCERT_KDB%" "%IBM_SP_WINPE_DIR%\media\Program Files 64\Tivoli\TSM\baclient\dsmcert.kdb"
            copy "%IBM_SP_DSMCERT_STH%" "%IBM_SP_WINPE_DIR%\media\Program Files 64\Tivoli\TSM\baclient\dsmcert.sth"
        ) else (
            copy "%IBM_SP_DSMCERT_KDB%" "%IBM_SP_WINPE_DIR%\media\Program Files\Tivoli\TSM\baclient\dsmcert.kdb"
            copy "%IBM_SP_DSMCERT_STH%" "%IBM_SP_WINPE_DIR%\media\Program Files\Tivoli\TSM\baclient\dsmcert.sth"
        )
    )

    REM Revert back to the deployment tools directory.
    popd

    REM Add the WMI image (if needed) and set the booted WinPE X: drive size to
    REM 512 MB
    REM Reference: for DISM return codes, see "C:\Program Files (x86)\Windows Kits\10\Assessment and Deployment Kit\Deployment Tools\SDKs\DismApi\Include\dismapi.h".
    echo.
    echo -------------------------------------------------------------------------------
    echo - Mount the image
    echo -------------------------------------------------------------------------------
    REM Troubleshooting tip: Use dism /get-mountedimageinfo to identify mounted
    REM images. Sometimes an image might not get unmounted correctly, and that
    REM can lead to anomalous dism behavior later on.
    dism /mount-wim /wimfile:"%IBM_SP_WINPE_DIR%\media\sources\boot.wim" /index:1 /mountdir:"%IBM_SP_WINPE_DIR%\mount"
    set IBM_SP_LAST_RC=%ERRORLEVEL%
    if %IBM_SP_LAST_RC% neq 0 goto :DISM_MOUNT_WIM_FAILED

    REM Add WMI, NetFx, and Scripting to the image.
    echo.
    echo -------------------------------------------------------------------------------
    echo - Add WMI to the image
    echo -------------------------------------------------------------------------------
    set IBM_SP_WINPE_PACKAGE_NAME=WinPE-WMI.cab
    dism /image:"%IBM_SP_WINPE_DIR%\mount" /add-package /packagepath:"..\windows preinstallation environment\%IBM_SP_WINPE_ARCH%\winpe_ocs\%IBM_SP_WINPE_PACKAGE_NAME%"
    set IBM_SP_LAST_RC=%ERRORLEVEL%
    if %IBM_SP_LAST_RC% neq 0 goto :DISM_ADD_WINPE_ADD_PACKAGE_FAILED

    set IBM_SP_WINPE_PACKAGE_NAME=WinPE-WMI_en-us.cab
    dism /image:"%IBM_SP_WINPE_DIR%\mount" /add-package /packagepath:"..\windows preinstallation environment\%IBM_SP_WINPE_ARCH%\winpe_ocs\en-us\%IBM_SP_WINPE_PACKAGE_NAME%"
    set IBM_SP_LAST_RC=%ERRORLEVEL%
    if %IBM_SP_LAST_RC% neq 0 goto :DISM_ADD_WINPE_ADD_PACKAGE_FAILED

    echo.
    echo -------------------------------------------------------------------------------
    echo - Add NetFx to the image
    echo -------------------------------------------------------------------------------
    set IBM_SP_WINPE_PACKAGE_NAME=WinPE-NetFx.cab
    dism /image:"%IBM_SP_WINPE_DIR%\mount" /add-package /packagepath:"..\windows preinstallation environment\%IBM_SP_WINPE_ARCH%\winpe_ocs\%IBM_SP_WINPE_PACKAGE_NAME%"
    set IBM_SP_LAST_RC=%ERRORLEVEL%
    if %IBM_SP_LAST_RC% neq 0 goto :DISM_ADD_WINPE_ADD_PACKAGE_FAILED

    set IBM_SP_WINPE_PACKAGE_NAME=WinPE-NetFx_en-us.cab
    dism /image:"%IBM_SP_WINPE_DIR%\mount" /add-package /packagepath:"..\windows preinstallation environment\%IBM_SP_WINPE_ARCH%\winpe_ocs\en-us\%IBM_SP_WINPE_PACKAGE_NAME%"
    set IBM_SP_LAST_RC=%ERRORLEVEL%
    if %IBM_SP_LAST_RC% neq 0 goto :DISM_ADD_WINPE_ADD_PACKAGE_FAILED

    echo.
    echo -------------------------------------------------------------------------------
    echo - Add HTA to the image
    echo -------------------------------------------------------------------------------
    set IBM_SP_WINPE_PACKAGE_NAME=WinPE-HTA.cab
    dism /image:"%IBM_SP_WINPE_DIR%\mount" /add-package /packagepath:"..\windows preinstallation environment\%IBM_SP_WINPE_ARCH%\winpe_ocs\%IBM_SP_WINPE_PACKAGE_NAME%"
    set IBM_SP_LAST_RC=%ERRORLEVEL%
    if %IBM_SP_LAST_RC% neq 0 goto :DISM_ADD_WINPE_ADD_PACKAGE_FAILED

    set IBM_SP_WINPE_PACKAGE_NAME=WinPE-HTA_en-us.cab
    dism /image:"%IBM_SP_WINPE_DIR%\mount" /add-package /packagepath:"..\windows preinstallation environment\%IBM_SP_WINPE_ARCH%\winpe_ocs\en-us\%IBM_SP_WINPE_PACKAGE_NAME%"
    set IBM_SP_LAST_RC=%ERRORLEVEL%
    if %IBM_SP_LAST_RC% neq 0 goto :DISM_ADD_WINPE_ADD_PACKAGE_FAILED

    echo.
    echo -------------------------------------------------------------------------------
    echo - Add Scripting to the image
    echo -------------------------------------------------------------------------------
    set IBM_SP_WINPE_PACKAGE_NAME=WinPE-Scripting.cab
    dism /image:"%IBM_SP_WINPE_DIR%\mount" /add-package /packagepath:"..\windows preinstallation environment\%IBM_SP_WINPE_ARCH%\winpe_ocs\%IBM_SP_WINPE_PACKAGE_NAME%"
    set IBM_SP_LAST_RC=%ERRORLEVEL%
    if %IBM_SP_LAST_RC% neq 0 goto :DISM_ADD_WINPE_ADD_PACKAGE_FAILED

    set IBM_SP_WINPE_PACKAGE_NAME=WinPE-Scripting_en-us.cab
    dism /image:"%IBM_SP_WINPE_DIR%\mount" /add-package /packagepath:"..\windows preinstallation environment\%IBM_SP_WINPE_ARCH%\winpe_ocs\en-us\%IBM_SP_WINPE_PACKAGE_NAME%"
    set IBM_SP_LAST_RC=%ERRORLEVEL%
    if %IBM_SP_LAST_RC% neq 0 goto :DISM_ADD_WINPE_ADD_PACKAGE_FAILED

    echo.
    echo -------------------------------------------------------------------------------
    echo - Add PowerShell to the image
    echo -------------------------------------------------------------------------------
    set IBM_SP_WINPE_PACKAGE_NAME=WinPE-PowerShell.cab
    dism /image:"%IBM_SP_WINPE_DIR%\mount" /add-package /packagepath:"..\windows preinstallation environment\%IBM_SP_WINPE_ARCH%\winpe_ocs\%IBM_SP_WINPE_PACKAGE_NAME%"
    set IBM_SP_LAST_RC=%ERRORLEVEL%
    if %IBM_SP_LAST_RC% neq 0 goto :DISM_ADD_WINPE_ADD_PACKAGE_FAILED

    set IBM_SP_WINPE_PACKAGE_NAME=WinPE-PowerShell_en-us.cab
    dism /image:"%IBM_SP_WINPE_DIR%\mount" /add-package /packagepath:"..\windows preinstallation environment\%IBM_SP_WINPE_ARCH%\winpe_ocs\en-us\%IBM_SP_WINPE_PACKAGE_NAME%"
    set IBM_SP_LAST_RC=%ERRORLEVEL%
    if %IBM_SP_LAST_RC% neq 0 goto :DISM_ADD_WINPE_ADD_PACKAGE_FAILED

    echo.
    echo -------------------------------------------------------------------------------
    echo - Add Storage WMI to the image
    echo -------------------------------------------------------------------------------
    set IBM_SP_WINPE_PACKAGE_NAME=WinPE-StorageWMI.cab
    dism /image:"%IBM_SP_WINPE_DIR%\mount" /add-package /packagepath:"..\windows preinstallation environment\%IBM_SP_WINPE_ARCH%\winpe_ocs\%IBM_SP_WINPE_PACKAGE_NAME%"
    set IBM_SP_LAST_RC=%ERRORLEVEL%
    if %IBM_SP_LAST_RC% neq 0 goto :DISM_ADD_WINPE_ADD_PACKAGE_FAILED

    set IBM_SP_WINPE_PACKAGE_NAME=WinPE-StorageWMI_en-us.cab
    dism /image:"%IBM_SP_WINPE_DIR%\mount" /add-package /packagepath:"..\windows preinstallation environment\%IBM_SP_WINPE_ARCH%\winpe_ocs\en-us\%IBM_SP_WINPE_PACKAGE_NAME%"
    set IBM_SP_LAST_RC=%ERRORLEVEL%
    if %IBM_SP_LAST_RC% neq 0 goto :DISM_ADD_WINPE_ADD_PACKAGE_FAILED

    REM Add user customizations (if any).
    REM Preserve the IBM_SP_WINPE_DIR environment variable just in case it is
    REM altered in the exit script (this should not happen!).
    REM Version 1.3: Added full DISM script path to the output below.
    echo.
    echo -------------------------------------------------------------------------------
    echo - Calling the DISM user exit "%IBM_SP_DISMEXIT_SCRIPT_FULL_PATH%"
    echo -------------------------------------------------------------------------------
    set IBM_SP_WINPE_DIR_BKUP=%IBM_SP_WINPE_DIR%
    call "%IBM_SP_DISMEXIT_SCRIPT_FULL_PATH%"
    set IBM_SP_LAST_RC=%ERRORLEVEL%
    if not "%IBM_SP_WINPE_DIR%"=="%IBM_SP_WINPE_DIR_BKUP%" goto :DISM_USER_EXIT_ALTERED_WINPE_DIR
    if %IBM_SP_LAST_RC% neq 0 goto :DISM_USER_EXIT_FAILED

    REM Sanity check: Show the package information.
    dism /image:"%IBM_SP_WINPE_DIR%\mount" /get-packages
    set IBM_SP_LAST_RC=%ERRORLEVEL%
    if %IBM_SP_LAST_RC% neq 0 goto :DISM_GET_PACKAGES_FAILED

    REM Sanity check: Show the driver information.
    dism /image:"%IBM_SP_WINPE_DIR%\mount" /get-drivers
    set IBM_SP_LAST_RC=%ERRORLEVEL%
    if %IBM_SP_LAST_RC% neq 0 goto :DISM_GET_DRIVERS_FAILED

    REM Set the size ("scratch space") of the booted WinPE X: drive to 512 MB
    REM (default is 32 MB).
    echo.
    echo -------------------------------------------------------------------------------
    echo - Set the X: drive size to 512 MB (default is 32 MB)
    echo -------------------------------------------------------------------------------
    dism /image:"%IBM_SP_WINPE_DIR%\mount" /set-scratchspace:512
    set IBM_SP_LAST_RC=%ERRORLEVEL%
    if %IBM_SP_LAST_RC% neq 0 goto :DISM_SET_SCRATCHSPACE_FAILED

    REM Sanity check: Show the image's scratch space size.
    dism /image:"%IBM_SP_WINPE_DIR%\mount" /get-scratchspace
    set IBM_SP_LAST_RC=%ERRORLEVEL%
    if %IBM_SP_LAST_RC% neq 0 goto :DISM_GET_SCRATCHSPACE_FAILED

    REM Version 1.6: Append code to startnet.cmd located in the WinPE image's
    REM Windows\System32 directory.
    echo.
    echo -------------------------------------------------------------------------------
    echo - Adding code to the WinPE startnet.cmd script to launch the restore script.
    echo -------------------------------------------------------------------------------
    call :MODIFY_STARTNET_CMD_SCRIPT

    REM We are done, so dismount the image.
    dism /unmount-wim /mountdir:"%IBM_SP_WINPE_DIR%\mount" /commit
    set IBM_SP_LAST_RC=%ERRORLEVEL%
    if %IBM_SP_LAST_RC% neq 0 goto :DISM_UNMOUNT_WIM_FAILED

    REM Now build the ISO image.
    echo.
    echo -------------------------------------------------------------------------------
    echo - Create the ISO image
    echo -------------------------------------------------------------------------------
    call makewinpemedia /iso "%IBM_SP_WINPE_DIR%" "%IBM_SP_WINPE_IMAGE%"
    set IBM_SP_LAST_RC=%ERRORLEVEL%
    if %IBM_SP_LAST_RC% neq 0 (
        if %IBM_SP_LAST_RC% neq 3 goto :MAKEWINPEMEDIA_FAILED
    )

    echo.
    echo -------------------------------------------------------------------------------
    echo - Success^^! ISO Image "%IBM_SP_WINPE_IMAGE%" creation
    echo - is complete
    echo -------------------------------------------------------------------------------
    dir "%IBM_SP_WINPE_IMAGE%"

    echo.
    echo -------------------------------------------------------------------------------
    echo - (Optional) You can create a bootable flash drive by running these commands:
    echo -
    echo -   "%IBM_SP_DEPLOYMENT_TOOLS_INIT_SCRIPT%"
    echo -   MakeWinPEMedia /UFD "%IBM_SP_WINPE_DIR%" ^<drive_letter^>
    echo -
    echo - where ^<drive_letter^> is the drive letter of the flash drive
    echo -
    echo - For example, if your flash drive is mounted on X:
    echo -
    echo -   MakeWinPEMedia /UFD "%IBM_SP_WINPE_DIR%" X:
    echo -
    echo - Caution: MakeWinPEMedia will overwrite the contents of the flash drive.
    echo -
    echo - Caution: Using a bootable flash drive is experimental and not formally
    echo -   supported. However, it is known to work in at least some cases for
    echo -   UEFI-based systems, but does not work for BIOS-based systems.
    echo -------------------------------------------------------------------------------

    goto :DONE


:MODIFY_STARTNET_CMD_SCRIPT
    set IBM_SP_STARTNET_SCRIPT=%IBM_SP_WINPE_DIR%\mount\Windows\System32\startnet.cmd
    echo.>>"%IBM_SP_STARTNET_SCRIPT%
    echo @echo off>>"%IBM_SP_STARTNET_SCRIPT%
    echo setlocal>>"%IBM_SP_STARTNET_SCRIPT%
    echo.>>"%IBM_SP_STARTNET_SCRIPT%
    echo set IBM_SP_RESTORE_SCRIPT_NAME=restore_system.cmd>>"%IBM_SP_STARTNET_SCRIPT%
    echo call :FIND_CD_DRIVE>>"%IBM_SP_STARTNET_SCRIPT%
    echo if "%%IBM_SP_CLIENT_DRIVE%%" equ "" goto :CANNOT_FIND_CD_DRIVE>>"%IBM_SP_STARTNET_SCRIPT%
    echo if not exist "%%IBM_SP_CLIENT_DRIVE%%\%%IBM_SP_RESTORE_SCRIPT_NAME%%" goto :CANNOT_FIND_RESTORE_SCRIPT>>"%IBM_SP_STARTNET_SCRIPT%
    echo.>>"%IBM_SP_STARTNET_SCRIPT%
    echo :ASK_TO_START_RESTORE_SCRIPT>>"%IBM_SP_STARTNET_SCRIPT%
    echo.>>"%IBM_SP_STARTNET_SCRIPT%
    echo echo.>>"%IBM_SP_STARTNET_SCRIPT%
    echo set /p IBM_SP_USER_RESP=Do you want to start the restore script (y/n) ? >>"%IBM_SP_STARTNET_SCRIPT%
    echo if /i "%%IBM_SP_USER_RESP%%" equ "y" goto :LAUNCH_RESTORE_SCRIPT>>"%IBM_SP_STARTNET_SCRIPT%
    echo if /i "%%IBM_SP_USER_RESP%%" equ "yes" goto :LAUNCH_RESTORE_SCRIPT>>"%IBM_SP_STARTNET_SCRIPT%
    echo if /i "%%IBM_SP_USER_RESP%%" equ "n" goto :SKIP_RESTORE_SCRIPT>>"%IBM_SP_STARTNET_SCRIPT%
    echo if /i "%%IBM_SP_USER_RESP%%" equ "no" goto :SKIP_RESTORE_SCRIPT>>"%IBM_SP_STARTNET_SCRIPT%
    echo echo.>>"%IBM_SP_STARTNET_SCRIPT%
    echo echo Reply with y for "yes" or n for "no">>"%IBM_SP_STARTNET_SCRIPT%
    echo echo.>>"%IBM_SP_STARTNET_SCRIPT%
    echo goto :ASK_TO_START_RESTORE_SCRIPT>>"%IBM_SP_STARTNET_SCRIPT%
    echo.>>"%IBM_SP_STARTNET_SCRIPT%
    echo :LAUNCH_RESTORE_SCRIPT>>"%IBM_SP_STARTNET_SCRIPT%
    echo "%%IBM_SP_CLIENT_DRIVE%%\%%IBM_SP_RESTORE_SCRIPT_NAME%%">>"%IBM_SP_STARTNET_SCRIPT%
    echo goto :DONE>>"%IBM_SP_STARTNET_SCRIPT%
    echo.>>"%IBM_SP_STARTNET_SCRIPT%
    echo :FIND_CD_DRIVE>>"%IBM_SP_STARTNET_SCRIPT%
    echo     set IBM_SP_CLIENT_DRIVE=>>"%IBM_SP_STARTNET_SCRIPT%
    echo     set IBM_SP_DUMMY_ID_FILE=_____I_AM_WINPE_BOOT_MEDIA_WITH_IBM_SPECTRUM_PROTECT_CLIENT______>>"%IBM_SP_STARTNET_SCRIPT%
    echo     for %%%%d in (A B C D E F G H I J K L M N O P Q R S T U V W X Y Z) do (>>"%IBM_SP_STARTNET_SCRIPT%
    echo         if exist "%%%%d:\%%IBM_SP_DUMMY_ID_FILE%%" (>>"%IBM_SP_STARTNET_SCRIPT%
    echo             set IBM_SP_CLIENT_DRIVE=%%%%d:>>"%IBM_SP_STARTNET_SCRIPT%
    echo         )>>"%IBM_SP_STARTNET_SCRIPT%
    echo     )>>"%IBM_SP_STARTNET_SCRIPT%
    echo.>>"%IBM_SP_STARTNET_SCRIPT%
    echo     goto :EOF>>"%IBM_SP_STARTNET_SCRIPT%
    echo.>>"%IBM_SP_STARTNET_SCRIPT%
    echo :CANNOT_FIND_CD_DRIVE>>"%IBM_SP_STARTNET_SCRIPT%
    echo     echo ERROR: The WinPE boot drive cannot be found. The restore_command.cmd script>>"%IBM_SP_STARTNET_SCRIPT%
    echo     echo        cannot be started.>>"%IBM_SP_STARTNET_SCRIPT%
    echo     goto :DONE>>"%IBM_SP_STARTNET_SCRIPT%
    echo.>>"%IBM_SP_STARTNET_SCRIPT%
    echo :CANNOT_FIND_RESTORE_SCRIPT>>"%IBM_SP_STARTNET_SCRIPT%
    echo     echo ERROR: %%IBM_SP_CLIENT_DRIVE%%:\%%IBM_SP_RESTORE_SCRIPT_NAME%% cannot be found.>>"%IBM_SP_STARTNET_SCRIPT%
    echo     goto :DONE>>"%IBM_SP_STARTNET_SCRIPT%
    echo.>>"%IBM_SP_STARTNET_SCRIPT%
    echo :SKIP_RESTORE_SCRIPT>>"%IBM_SP_STARTNET_SCRIPT%
    echo     echo OK. If you change your mind, run %%IBM_SP_CLIENT_DRIVE%%\%%IBM_SP_RESTORE_SCRIPT_NAME%%>>"%IBM_SP_STARTNET_SCRIPT%
    echo     goto :DONE>>"%IBM_SP_STARTNET_SCRIPT%
    echo.>>"%IBM_SP_STARTNET_SCRIPT%
    echo :DONE>>"%IBM_SP_STARTNET_SCRIPT%
    echo     endlocal>>"%IBM_SP_STARTNET_SCRIPT%
    echo     goto :EOF>>"%IBM_SP_STARTNET_SCRIPT%

    goto :EOF


:NOT_ELEVATED_ADMIN
    echo -------------------------------------------------------------------------------
    echo - ERROR: This script must be run by a local administrator from a command prompt
    echo -        with elevated privileges. Run '%IBM_SP_SCRIPT_NAME% -?' for
    echo -        additional usage information.
    if "%IBM_SP_DEBUG%" == "true" (
        echo -
        echo - DEBUG: Displaying output from "whoami /groups" command
        whoami /groups
    )
    echo -------------------------------------------------------------------------------
    goto :DONE


:DEPLOY_TOOLS_NOT_INSTALLED
    echo -------------------------------------------------------------------------------
    echo - ERROR: This script cannot find the installation directory for the Windows 10
    echo -        ADK. Verify that the Windows 10 ADK is installed correctly, and then
    echo -        try this script again.
    echo -------------------------------------------------------------------------------
    goto :DONE


:DEPLOY_TOOLS_INIT_NOT_FOUND
    echo -------------------------------------------------------------------------------
    echo - ERROR: This script cannot find the Windows 10 ADK command shell
    echo -        initialization script:
    echo -
    echo -          "%IBM_SP_DEPLOYMENT_TOOLS_INIT_SCRIPT%".
    echo -
    echo -        Verify that the Windows 10 ADK is installed correctly, and then try
    echo -        this script again.
    echo -------------------------------------------------------------------------------
    goto :DONE


:DEPLOY_TOOLS_STATE_INDETERMINATE
    echo -------------------------------------------------------------------------------
    echo - ERROR: This script detected a partially, but not fully, initialized Windows
    echo -        ADK command shell environment. Possible reasons include, but are not
    echo -        necessarily limited to:
    echo -
    echo -          * The Windows 10 ADK command shell was started, but changes were
    echo -            made to the current directory or environment variables before
    echo -            starting this script.
    echo -
    echo -          * The Windows 10 ADK installation is incomplete.
    echo -
    echo -        Verify that the Windows 10 ADK is installed correctly, and then try
    echo -        this script again. You can run this script from a normal Windows
    echo -        command prompt without any ADK initialization, or you can start the
    echo -        Windows 10 ADK command shell, and then run this script without
    echo -        changing the current directory or any environment variables.
    echo -------------------------------------------------------------------------------
    goto :DONE


:CANNOT_INITIALIZE_DEPLOYMENT_TOOLS
    echo -------------------------------------------------------------------------------
    echo - ERROR: The Windows 10 ADK tools deployment shell did not seem to be running
    echo -        when this script was started, and this script was unable to
    echo -        successfully initialize the shell.
    echo -
    echo -        Verify that the Windows 10 ADK is installed correctly, and then try
    echo -        this script again. You can run this script from a normal Windows
    echo -        command prompt without any ADK initialization, or you can start the
    echo -        Windows 10 ADK command shell, and then run this script without
    echo -        changing the current directory or any environment variables.
    echo -------------------------------------------------------------------------------
    goto :DONE


:COULD_NOT_CLEAN_IBMSPPKGDIR
    echo.
    echo -------------------------------------------------------------------------------
    echo - ERROR: Could not clean directory '%IBM_SP_PKG_DIR%'
    echo -------------------------------------------------------------------------------
    goto :DONE


:PACKAGE_NOT_SPECIFIED
    echo.
    echo -------------------------------------------------------------------------------
    echo - ERROR: Package name was not specified.
    echo -
    echo - Run '%IBM_SP_SCRIPT_NAME% -?' for additional usage information.
    echo -------------------------------------------------------------------------------
    goto :DONE


:TOO_MANY_ARGS
    set IBM_SP_EXTRANEOUS_ARGS=%2
    :EXTRANEOUS_ARGS_LOOP
        shift /2
        if "%2" equ "" goto :END_EXTRANEOUS_ARGS_LOOP
        set IBM_SP_EXTRANEOUS_ARGS=%IBM_SP_EXTRANEOUS_ARGS% %2
        goto :EXTRANEOUS_ARGS_LOOP
    :END_EXTRANEOUS_ARGS_LOOP
    echo.
    echo -------------------------------------------------------------------------------
    echo - ERROR:     Extraneous argument(s) "%IBM_SP_EXTRANEOUS_ARGS%"
    echo -            detected after package name "%1"
    echo -
    echo - Run '%IBM_SP_SCRIPT_NAME% -?' for additional usage information.
    echo -------------------------------------------------------------------------------
    goto :DONE


:PACKAGE_NOT_FOUND
    echo.
    echo -------------------------------------------------------------------------------
    echo - ERROR: Backup-archive client package '%1' cannot be found.
    echo -
    echo - Run '%IBM_SP_SCRIPT_NAME% -?' for additional usage information.
    echo -------------------------------------------------------------------------------
    goto :DONE


:DISMEXIT_NOT_FOUND
    echo.
    echo -------------------------------------------------------------------------------
    echo - ERROR: DISM user exit script '%IBM_SP_DISMEXIT_SCRIPT_FULL_PATH%'
    echo -        cannot be found. This script is expected to exist, even if it is not
    echo -        customized.
    echo -------------------------------------------------------------------------------
    goto :DONE


:RESTORE_SCRIPT_NOT_FOUND
    REM Version 1.6
    echo.
    echo -------------------------------------------------------------------------------
    echo - ERROR: Restore script '%IBM_SP_RESTORE_SYSTEM_SCRIPT_FULL_PATH%'
    echo -        cannot be found. This script is expected to exist, even if it is not
    echo -        used during the restore.
    echo -------------------------------------------------------------------------------
    goto :DONE


:CERT_FILES_NOT_FOUND
    REM Version 1.6
    echo.
    echo -------------------------------------------------------------------------------
    echo - ERROR: -addcert was specified but one or both of these files is missing:
    echo -
    echo -            "%IBM_SP_DSMCERT_KDB%"
    echo -            "%IBM_SP_DSMCERT_STH%"
    echo -
    echo -        Copy your dsmcert.kdb and dsmcert.sth files to directory
    echo -        %IBM_SP_SCRIPT_DRIVE%%IBM_SP_SCRIPT_DIR%,
    echo -        then run this script again.
    echo -------------------------------------------------------------------------------
    goto :DONE


:DEPLOY_TOOLS_NOT_FOUND
    echo.
    echo -------------------------------------------------------------------------------
    echo - ERROR: This script cannot determine if it is running in the Windows 10 ADK
    echo -        command shell. Open the "Win 10 ADK Deployment and Imaging Tools
    echo -        Environment" command shell (making sure to run it as an administrator
    echo -        if UAC is enabled), then run this script from within that command
    echo -        shell. Run '%IBM_SP_SCRIPT_NAME% -?' for additional usage
    echo -        information.
    echo -------------------------------------------------------------------------------
    goto :DONE


:EXTRACT_FAILED
    echo.
    echo -------------------------------------------------------------------------------
    echo - ERROR: '"%1" -o"%IBM_SP_PKG_DIR%"'
    echo -        failed to extract the backup-archive client package, return code %IBM_SP_LAST_RC%.
    echo -------------------------------------------------------------------------------
    goto :DONE


:COPY_PE_FAILED
    echo.
    echo -------------------------------------------------------------------------------
    echo - ERROR: copype command failed with return code %IBM_SP_LAST_RC%.
    echo -------------------------------------------------------------------------------
    goto :DONE


:WINPE_DIR_NOT_CREATED
    echo.
    echo -------------------------------------------------------------------------------
    echo - ERROR: copype command ran successfully, but directory "%IBM_SP_WINPE_DIR%"
    echo          could not be found.
    echo -------------------------------------------------------------------------------
    goto :DONE


:COULD_NOT_CHANGE_TO_TSMCLIENT_DIR
    echo.
    echo -------------------------------------------------------------------------------
    echo - ERROR: Attempt to change to "%IBM_SP_PKG_DIR%\TSMClient" failed.
    echo -------------------------------------------------------------------------------
    goto :DONE


:ROBOCOPY_FAILED
    echo.
    echo -------------------------------------------------------------------------------
    echo - ERROR: ROBOCOPY command failed with return code %IBM_SP_LAST_RC%
    echo -        For information about ROBOCOPY return codes, see
    echo -        https://support.microsoft.com/en-us/help/954404/return-codes-that-are-used-by-the-robocopy-utility-in-windows-server-2
    echo -------------------------------------------------------------------------------
    goto :DONE


:DISM_MOUNT_WIM_FAILED
    echo.
    echo -------------------------------------------------------------------------------
    echo - ERROR: dism /mount-wim command ended with return code %IBM_SP_LAST_RC%
    echo -------------------------------------------------------------------------------
    goto :DONE


:DISM_ADD_WINPE_ADD_PACKAGE_FAILED
    echo.
    echo -------------------------------------------------------------------------------
    echo - ERROR: dism /add-package for %IBM_SP_WINPE_PACKAGE_NAME% ended with return code %IBM_SP_LAST_RC%
    echo -        Attempting to unmount the image...
    echo -------------------------------------------------------------------------------
    dism /unmount-wim /mountdir:"%IBM_SP_WINPE_DIR%\mount" /discard
    set IBM_SP_LAST_RC=%ERRORLEVEL%
    if %IBM_SP_LAST_RC% neq 0 goto :DISM_UNMOUNT_WIM_FAILED
    goto :DONE


:DISM_GET_PACKAGES_FAILED
    echo.
    echo -------------------------------------------------------------------------------
    echo - ERROR: dism /get-packages ended with return code %IBM_SP_LAST_RC%
    echo -        Attempting to unmount the image...
    echo -------------------------------------------------------------------------------
    dism /unmount-wim /mountdir:"%IBM_SP_WINPE_DIR%\mount" /discard
    set IBM_SP_LAST_RC=%ERRORLEVEL%
    if %IBM_SP_LAST_RC% neq 0 goto :DISM_UNMOUNT_WIM_FAILED
    goto :DONE


:DISM_GET_DRIVERS_FAILED
    echo.
    echo -------------------------------------------------------------------------------
    echo - ERROR: dism /get-drivers ended with return code %IBM_SP_LAST_RC%
    echo -        Attempting to unmount the image...
    echo -------------------------------------------------------------------------------
    dism /unmount-wim /mountdir:"%IBM_SP_WINPE_DIR%\mount" /discard
    set IBM_SP_LAST_RC=%ERRORLEVEL%
    if %IBM_SP_LAST_RC% neq 0 goto :DISM_UNMOUNT_WIM_FAILED
    goto :DONE


:DISM_USER_EXIT_ALTERED_WINPE_DIR
    echo.
    echo -------------------------------------------------------------------------------
    echo - ERROR: The DISM custom user exit %IBM_SP_DISMEXIT_SCRIPT_FULL_PATH%
    echo -        changed IBM_SP_WINPE_DIR from "%IBM_SP_WINPE_DIR_BKUP%" to
    echo -        "%IBM_SP_WINPE_DIR%%". This is not allowed, and is considered
    echo -        an error condition.
    echo -        Attempting to unmount the image...
    echo -------------------------------------------------------------------------------
    set IBM_SP_WINPE_DIR=%IBM_SP_WINPE_DIR_BKUP%
    dism /unmount-wim /mountdir:"%IBM_SP_WINPE_DIR%\mount" /discard
    set IBM_SP_LAST_RC=%ERRORLEVEL%
    if %IBM_SP_LAST_RC% neq 0 goto :DISM_UNMOUNT_WIM_FAILED
    goto :DONE


:DISM_USER_EXIT_FAILED
    echo.
    echo -------------------------------------------------------------------------------
    echo - ERROR: The DISM custom user exit %IBM_SP_DISMEXIT_SCRIPT_FULL_PATH%
    echo -        ended with return code %IBM_SP_LAST_RC%
    echo -        Attempting to unmount the image...
    echo -------------------------------------------------------------------------------
    dism /unmount-wim /mountdir:"%IBM_SP_WINPE_DIR%\mount" /discard
    set IBM_SP_LAST_RC=%ERRORLEVEL%
    if %IBM_SP_LAST_RC% neq 0 goto :DISM_UNMOUNT_WIM_FAILED
    goto :DONE


:DISM_SET_SCRATCHSPACE_FAILED
    echo.
    echo -------------------------------------------------------------------------------
    echo - ERROR: dism /set-scratchspace ended with return code %IBM_SP_LAST_RC%
    echo -        Attempting to unmount the image...
    echo -------------------------------------------------------------------------------
    dism /unmount-wim /mountdir:"%IBM_SP_WINPE_DIR%\mount" /discard
    set IBM_SP_LAST_RC=%ERRORLEVEL%
    if %IBM_SP_LAST_RC% neq 0 goto :DISM_UNMOUNT_WIM_FAILED
    goto :DONE


:DISM_GET_SCRATCHSPACE_FAILED
    echo.
    echo -------------------------------------------------------------------------------
    echo - ERROR: dism /get-scratchspace ended with return code %IBM_SP_LAST_RC%
    echo -        Attempting to unmount the image...
    echo -------------------------------------------------------------------------------
    dism /unmount-wim /mountdir:"%IBM_SP_WINPE_DIR%\mount" /discard
    set IBM_SP_LAST_RC=%ERRORLEVEL%
    if %IBM_SP_LAST_RC% neq 0 goto :DISM_UNMOUNT_WIM_FAILED
    goto :DONE


:DISM_UNMOUNT_WIM_FAILED
    echo.
    echo -------------------------------------------------------------------------------
    echo - ERROR: dism /unmount-wim command ended with return code %IBM_SP_LAST_RC%
    echo -------------------------------------------------------------------------------
    goto :DONE


:MAKEWINPEMEDIA_FAILED
    echo.
    echo -------------------------------------------------------------------------------
    echo - ERROR: MakeWinPEMedia command failed with return code %IBM_SP_LAST_RC%
    echo -------------------------------------------------------------------------------
    goto :DONE


:BAD_VERSION
    echo.
    echo -------------------------------------------------------------------------------
    echo - ERROR: The version information derived from the package name is either
    echo -        invalid or not supported by this script. This script supports these
    echo -        versions:
    echo -
    echo -        7.1.6.0 and later 7.1.x levels
    echo -        8.1.0.0 and later versions and levels
    echo -------------------------------------------------------------------------------
    goto :DONE


:BAD_PACKAGE_NAME
    echo.
    echo -------------------------------------------------------------------------------
    echo - ERROR: The package name "%IBM_SP_PKG_NAME%" is not valid.
    echo -        The expected format is:
    echo -
    echo -        ^<v^>.^<r^>.^<m^>.^<f^>-TIV-TSMBAC-WinX^<nn^>.exe
    echo -
    echo -        where '^<v^>' is the version, '^<r^>' is the release, '^<m^>' is the
    echo -        maintenance (fix pack) level, '^<f^>' is the interim fix level, '^<nn^>'
    echo -        is 32 or 64 (processor architecture), and the other parts are literal
    echo -        text.
    echo -
    echo - Example: 7.1.8.3-TIV-TSMBAC-WinX64.exe
    echo -------------------------------------------------------------------------------
    goto :DONE


:INTERNAL_ERROR
    echo.
    echo -------------------------------------------------------------------------------
    echo - ERROR: An internal error was detected:
    echo -
    echo -  %IBM_SP_NUM_TESTS_PASSED%
    echo -------------------------------------------------------------------------------
    goto :DONE


:USAGE
    REM Use '^' as escape character.
    echo.
    echo -------------------------------------------------------------------------------
    echo - Usage: %IBM_SP_SCRIPT_NAME% [-debug] [-addcert] ^<ftp_package_name^>
    echo - Usage: %IBM_SP_SCRIPT_NAME% -?
    echo -
    echo -        -?   Displays this usage information.
    echo -
    echo -        -debug   Enables verbose display of the script operations. For best
    echo -        results, redirect stdout and stderr to a file.
    echo -
    echo -        -addcert   Adds the IBM Spectrum Protect server public certificate
    echo -        files dsmcert.kdb and dsmcert.sth to the WinPE image. This option
    echo -        requires that the files are available in the same directory as this
    echo -        script.
    echo -
    echo -        ^<ftp_package_name^>   the name of a backup-archive install package
    echo -        downloaded from the FTP site or Fix Central. This script supports
    echo -        these client versions:
    echo -
    echo -        7.1.6.0 and later 7.1.x levels
    echo -        8.1.0.0 and later versions and levels
    echo -
    echo -        Example: %IBM_SP_SCRIPT_NAME% 8.1.4.1-TIV-TSMBAC-WinX64.exe
    echo -
    echo - This script requires the Microsoft Assessment and Deployment Kit (ADK)
    echo - version 10. If the ADK is not already installed, you can find it at
    echo - https://support.microsoft.com. Search for "windows adk for windows 10"
    echo - (without the quotation marks). Contact Microsoft support if you need further
    echo - assistance with installing and using the ADK. Important: Do not use ADK
    echo - version 1803. Refer to Microsoft knowledge base article KB4458469 and IBM
    echo - APAR IT26393 for additional information.
    echo -
    echo - To run the script:
    echo -
    echo -     1. Log in as a member of the Administrators group.
    echo -
    echo -     2. Start a Windows command prompt (cmd.exe). If Windows UAC (User Account
    echo -        Control) is enabled, the prompt must be started with the "Run as
    echo -        administrator" option.
    echo -
    echo -     3. Start this script as described in "Usage" above.
    echo -
    echo - Tip: If you need to include additional features or drivers in the WinPE
    echo -      image, see user exit script build_recovery_image_dismexit.cmd.
    echo -
    echo - Additional reference information:
    echo -     https://ibm.biz/build_recovery_image
    echo -     https://ibm.biz/system_state_backup_recovery_guidelines
    echo -------------------------------------------------------------------------------
    echo.

    goto :DONE


:DONE
    endlocal
